VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Amortization"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private m_objPaymentDetails As Payments
Private m_objExtraPaymentDetails As Payments
Private m_curAmount As Currency
Private m_dblRate As Double
Private m_lngNumPayments As Long
Private m_lngPaymentsPerYear As Long
Private m_curExtraPayment As Currency
Private m_curPaymentAmount As Currency
Private m_curTotalInterest As Currency
Private m_curExtraInterest As Currency

Private Const INVALIDAMOUNTERR As Long = vbObjectError + 1001
Private Const INVALIDAMOUNT As String = _
    "The amount of the loan must be specified"
Private Const INVALIDRATEERR As Long = vbObjectError + 1002
Private Const INVALIDRATE As String = _
    "The specified interest rate is invalid"
Private Const INVALIDNUMPAYMENTSERR As Long = vbObjectError + 1003
Private Const INVALIDNUMPAYMENTS As String = _
    "The number of payments must be greater than 0"
Private Const INVALIDPAYMENTSPERERR As Long = vbObjectError + 1004
Private Const INVALIDPAYMENTSPER As String = _
    "There must be at least one payment due each year"

' Create class event handlers

Private Sub Class_Initialize()
    ' create the Payments objects that will maintain the list
    ' of information about each payment
    Set m_objPaymentDetails = New Payments
    Set m_objExtraPaymentDetails = New Payments
End Sub

Private Sub Class_Terminate()
    ' ensure that the memory for the objects is freed
    Set m_objPaymentDetails = Nothing
    Set m_objExtraPaymentDetails = Nothing
End Sub

' Create the properties for our class

' the Amount property indicates the amount of money borrowed
Public Property Get Amount() As Currency
    Amount = m_curAmount
End Property

Public Property Let Amount(curAmount As Currency)
    ' check for valid property value
    If curAmount <= 0 Then
        Err.Raise vbObjectError + 1001, _
    "FinancialUtility:Amortization:Amount", INVALIDAMOUNT
    End If
    m_curAmount = curAmount
End Property

' the Rate property is the interest rate for the loan
' it is expressed in percentages - for example 6.5 is 6.5% or .065 per year
Public Property Get Rate() As Double
    Rate = m_dblRate
End Property

Public Property Let Rate(dblRate As Double)
    ' check for valid property value
    If dblRate <= 0 Or dblRate > 50 Then
        Err.Raise INVALIDRATEERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDRATE
    End If
    m_dblRate = dblRate
End Property

' the NumberOfPayments property identifies the number or
' payments that are scheduled to be made over the life of
' the loan
Public Property Get NumberOfPayments() As Long
    NumberOfPayments = m_lngNumPayments
End Property

Public Property Let NumberOfPayments(lngPayments As Long)
    ' check for valid property value
    If lngPayments <= 0 Then
        Err.Raise INVALIDNUMPAYMENTSERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDNUMPAYMENTS
    End If
    m_lngNumPayments = lngPayments
End Property

' the PaymentsPerYear property identifies the number of payments
' that are required each year - used to calculate the periodic rate
Public Property Get PaymentsPerYear() As Long
    PaymentsPerYear = m_lngPaymentsPerYear
End Property

Public Property Let PaymentsPerYear(lngPaymentsPerYear As Long)
    ' check for valid property value
    If lngPaymentsPerYear <= 0 Then
        Err.Raise INVALIDPAYMENTSPERERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDPAYMENTSPER
    End If
    m_lngPaymentsPerYear = lngPaymentsPerYear
End Property

' the ExtraPayment property is included to show how adding an
' extra amount of principal to each payment can affect the
' total cost of borrowing the money
Public Property Get ExtraPayment() As Currency
    ExtraPayment = m_curExtraPayment
End Property

Public Property Let ExtraPayment(curExtraPayment As Currency)
    m_curExtraPayment = curExtraPayment
End Property

' the PaymentAmount property is set when the calculate method
' is called - this is a read only property that tells the
' required periodic payment for the loan
Public Property Get PaymentAmount() As Currency
    PaymentAmount = m_curPaymentAmount
End Property

' the TotalInterest property is set when the calculate method
' is called - this is a read only property that tells the
' total amount of interest to be paid over the life of the loan
Public Property Get TotalInterest() As Currency
    TotalInterest = m_curTotalInterest
End Property

' the TotalInterestWithExtra property is set when the calculate
' method is called, but it is only set if the ExtraPayment property
' is greater than 0 - this is a read only property that tells the
' total amount of interest to be paid over the life of the loan
' if the specified extra payment is made
Public Property Get TotalInterestWithExtra() As Currency
    If m_curExtraPayment > 0 Then
        TotalInterestWithExtra = m_curExtraInterest
    Else
        TotalInterestWithExtra = TotalInterest
    End If
End Property

' the TotalNumPaymentsWithExtra property is set when the calculate
' method is called, but it is only set if the ExtraPayment property
' is greater than 0 - this is a read only property that tells the
' total number of payments to be made if the specified extra payment
' is made
Public Property Get TotalNumPaymentsWithExtra() As Long
    If m_curExtraPayment > 0 Then
        TotalNumPaymentsWithExtra = ExtraPaymentDetails.Count
    Else
        TotalNumPaymentsWithExtra = PaymentDetails.Count
    End If
End Property

' the PaymentDetails property returns a reference to a Payments
' object that hold information about each scheduled payment
' for the loan
Public Property Get PaymentDetails() As Payments
    Set PaymentDetails = m_objPaymentDetails
End Property

' the ExtraPaymentDetails property returns a reference to a Payments
' object that hold information about each scheduled payment
' for the loan - this is only filled if there are extra payments
' to be made, and it reflects the effects of these payments
Public Property Get ExtraPaymentDetails() As Payments
    Set ExtraPaymentDetails = m_objExtraPaymentDetails
End Property

' Create the methods for our class

' the calculate method is the only method for our class
' but it is the work horse where all of the information is
' generated
' it accepts optional parameters and sets the properties
' accordingly if they are passed in
Public Sub Calculate(Optional curAmount As Currency, _
Optional dblRate As Double, Optional lngNumPayments As Long, _
Optional lngPaymentsPerYear As Long, Optional curExtraPayment As Currency)

    ' declare local variables
    Dim dblPeriodicRate As Double
    Dim dblFactor2 As Double
    Dim dblFactor4 As Double
    Dim curCurrentBalance As Currency
    Dim lngCounter As Long
    Dim objPayment As Payment

    On Error GoTo CALCULATE_ERROR

    ' check to see if we need to set the properties to the input values
    If curAmount <> 0 Then m_curAmount = curAmount
    If dblRate <> 0 Then m_dblRate = dblRate
    If lngNumPayments <> 0 Then m_lngNumPayments = lngNumPayments
    If lngPaymentsPerYear <> 0 Then _
    m_lngPaymentsPerYear = lngPaymentsPerYear
    If curExtraPayment <> 0 Then m_curExtraPayment = curExtraPayment

    ' ensure that we have the required information or send an
    ' error to the user
    If m_curAmount <= 0 Then
        Err.Raise INVALIDAMOUNTERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDAMOUNT
    End If
    
    If m_dblRate <= 0 Or dblRate > 50 Then
        Err.Raise INVALIDRATEERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDRATE
    End If
    
    If m_lngNumPayments <= 0 Then
        Err.Raise INVALIDNUMPAYMENTSERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDNUMPAYMENTS
    End If
    
    If m_lngPaymentsPerYear <= 0 Then
        Err.Raise INVALIDPAYMENTSPERERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDPAYMENTSPER
    End If
    
    ' calculate the periodic interest rate based on the rate
    ' and number of payments per year
    dblPeriodicRate = (m_dblRate / 100 / m_lngPaymentsPerYear)

    ' use the periodic rate and other inputs to calculate
    ' the required payment.  This is broken into steps
    ' to facilitate better understanding
    dblFactor2 = 1 / (1 + dblPeriodicRate) ^ m_lngNumPayments
    dblFactor4 = (1 - dblFactor2) / dblPeriodicRate
    m_curPaymentAmount = m_curAmount / dblFactor4

    ' compute each payment for the loan
    ' keep track of the current balance so we can
    ' identify each payments details
    curCurrentBalance = m_curAmount

    For lngCounter = 1 To m_lngNumPayments
        ' for each payment create a Payment object
        ' and set its properties
        Set objPayment = New Payment

        With objPayment
            .PaymentNumber = lngCounter
            .PaymentAmount = m_curPaymentAmount
            .Interest = curCurrentBalance * dblPeriodicRate
            ' keep a running total of overall interest paid
            m_curTotalInterest = m_curTotalInterest + .Interest
            .Principal = m_curPaymentAmount - .Interest
            .RemainingBalance = curCurrentBalance - .Principal

            curCurrentBalance = .RemainingBalance
        End With

        ' add the new payment to the Payments collection
        m_objPaymentDetails.Add objPayment
        ' free our object - VB does not always work
        ' correctly if we don't
        Set objPayment = Nothing
    Next

    ' compute each payment for the loan using the ExtraPayment value
    If m_curExtraPayment > 0 Then
        ' keep track of the current balance so we can
        ' identify each payments details
        curCurrentBalance = m_curAmount
        lngCounter = 1

        While curCurrentBalance > 0
            ' for each payment create a Payment object
            ' and set its properties
            Set objPayment = New Payment
    
            With objPayment
                .PaymentNumber = lngCounter
                .Interest = curCurrentBalance * dblPeriodicRate
                ' keep a running total of overall interest paid
                m_curExtraInterest = m_curExtraInterest + .Interest
                .Principal = _
                m_curPaymentAmount - .Interest + m_curExtraPayment
                .RemainingBalance = curCurrentBalance - .Principal
    
                curCurrentBalance = .RemainingBalance
            End With
    
            ' add the new payment to the Payments collection
            m_objExtraPaymentDetails.Add objPayment
            ' free our object - VB does not always
            ' work correctly if we don't
            Set objPayment = Nothing
        Wend
    End If

    Exit Sub

CALCULATE_ERROR:
    ' if required add specific error handling here
    ' for now we will just raise all errors to the calling
    ' program
    Err.Raise Err.Number

End Sub

' Create supporting procedures for our class
' N/A


